#!/usr/bin/perl -I/usr/sausalito/perl
# $Id: /usr/sausalito/sbin/convert_venet_to_incus.pl
#
# This script converts the network settings of an imported OpenVZ 7
# container to regular eth0 format as used on Incus.
#
# For more information see:
#
# https://www.aventurin.net/index.php?page=import-5210r-from-openvz-7
#

use CCE;
$cce = new CCE;
$cce->connectuds();

# Root check:
$id = `id -u`;
chomp($id);
if ($id ne "0") {
    print "$0 must be run by user 'root'!\n";
    $cce->bye('FAIL');
    exit(1);
}

if (! -e '/dev/incus/sock') {
    print "This script should only be run in an Incus instance that has been imported from OpenVZ 7!\n";
    $cce->bye('FAIL');
    exit(1);
}

# Find all 'Network' objects:
@Network = ();
(@Network) = $cce->findx('Network');

# Start sane:
$found = "0";
$eth0_oid = '';
$pri_net_oid = '';
$need_conversion = 0;

# Check if this Network object for eth0 already exists:
@oids = $cce->find('Network', {'device' => 'eth0'});
if ($#oids == 0) {
    $eth0_oid = $oids[0];
}

# Check if this Network object for venet0:0 exists:
@oids = $cce->find('Network', {'device' => 'venet0:0'});
if ($#oids == 0) {
    $pri_net_oid = $oids[0];
}

if (($eth0_oid ne '') && ($pri_net_oid ne '')) {
    $need_conversion = 1;
}

if ($need_conversion == 0) {
    print "Nothing to do! Exiting.\n";
    $cce->bye('FAIL');
    exit(1);
}

print "Converting Network Obkects from OpenVZ 7 format to Incus format.\n\n";

print "CODB OID of 'eth0': $eth0_oid\n";
print "CODB OID OID 'venet0:0': $pri_net_oid\n";
print "Need conversion: $need_conversion\n";

print "Editing /etc/sysconfig/network\n";
system("sed -i 's/^GATEWAY=venet0/GATEWAY=eth0/' /etc/sysconfig/network");
print "\nDeleting /etc/sysconfig/network-scripts/ifcfg-venet*\n";
system("rm -f /etc/sysconfig/network-scripts/ifcfg-venet*");

# Walk through all Network Objects:
for $net_object (@Network) {

    ($ok, $my_net) = $cce->get($net_object);

    if ($need_conversion == 1) {
        print "Processing Device " . $my_net->{'device'} . "\n";
    }

    if ($my_net->{'device'} eq 'venet0') {
        ($ok) = $cce->set($net_object, '', { 'enabled' => '0', 'real' => '0' });
        ($ok) = $cce->destroy($net_object);
    }
    elsif ($my_net->{'device'} eq 'venet0:0') {
        if ($need_conversion == 1) {
            # Destroy real eth0;
            ($ok) = $cce->destroy($eth0_oid);
        }
        ($ok) = $cce->set($net_object, '', { 'enabled' => '1', 'real' => '1', 'device' => 'eth0', 'netmask' => '255.255.255.240' });
    }
    else {
        ($ok) = $cce->set($net_object, '', { 'enabled' => '0', 'real' => '0', 'ipaddr' => '127.0.0.10', 'ipaddr_IPv6' });
        ($ok) = $cce->destroy($net_object);
    }
}

if ($need_conversion == 1) {
    my ($oid) = $cce->find('System');
    if ($oid) {
        print "\nUpdating 'System' object\n";
        ($ok) = $cce->set($oid, '', { 'nw_update' => time() });
        print "\nIMPORTANT: Please run /root/network_settings.sh to configure the network.\n";
    }
}

# tell cce everything is okay
$cce->bye('SUCCESS');
exit(0);

# 
# Copyright (c) 2024 Michael Stauber, SOLARSPEED.NET
# Copyright (c) 2024 Team BlueOnyx, BLUEONYX.IT
# All Rights Reserved.
# 
# 1. Redistributions of source code must retain the above copyright 
#     notice, this list of conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright 
#     notice, this list of conditions and the following disclaimer in 
#     the documentation and/or other materials provided with the 
#     distribution.
# 
# 3. Neither the name of the copyright holder nor the names of its 
#     contributors may be used to endorse or promote products derived 
#     from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.
# 
# You acknowledge that this software is not designed or intended for 
# use in the design, construction, operation or maintenance of any 
# nuclear facility.
#